declare module 'typedi/Handlers' {
	/**
	 * Storage value to be injected into constructor parameter.
	 */
	export interface ParamHandler {
	    type: Function;
	    index: number;
	    getValue: () => any;
	}
	/**
	 * Storage value to be injected into class property.
	 */
	export interface PropertyHandler {
	    target: Function;
	    key: string;
	    getValue: () => any;
	}

}
declare module 'typedi/Container' {
	import { ParamHandler, PropertyHandler } from 'typedi/Handlers';
	/**
	 * Service container.
	 */
	export class Container {
	    private static instances;
	    private static paramHandlers;
	    private static propertyHandlers;
	    private static registeredServices;
	    /**
	     * Registers a new constructor parameter handler.
	     */
	    static registerParamHandler(paramHandler: ParamHandler): void;
	    /**
	     * Registers a new class property handler.
	     */
	    static registerPropertyHandler(propertyHandler: PropertyHandler): void;
	    /**
	     * Registers a new service.
	     *
	     * @param name Service name. Optional
	     * @param type Service class
	     * @param params Parameters to be sent to constructor on service initialization
	     */
	    static registerService(name: string, type: Function, params?: any[]): void;
	    /**
	     * Retrieves the service with the specific name or given type from the service container.
	     * Optionally parameters can be pass in the case if instance is initialized in the container for the first time.
	     */
	    static get<T>(type: Function, params?: any[]): T;
	    static get<T>(name: string, params?: any[]): T;
	    /**
	     * Sets a value for the given type or service name in the container.
	     */
	    static set(type: Function, value: any): void;
	    static set(name: string, type: Function, value: any): void;
	    /**
	     * Provides a set of values to be saved in the container.
	     */
	    static provide(values: {
	        name?: string;
	        type: Function;
	        value: any;
	    }[]): void;
	    private static applyPropertyHandlers(target);
	    private static findInstance(name, type);
	    private static findInstanceByName(name);
	    private static findInstanceByType(type);
	    private static findRegisteredService(name, type);
	    private static findRegisteredServiceByType(type);
	    private static findRegisteredServiceByName(name);
	    private static findParamHandler(type, index);
	    private static initializeParams(type, params);
	    private static isTypeSimple(param);
	}

}
declare module 'typedi/Decorators' {
	/**
	 * Marks class as a service that can be injected using container.
	 *
	 * @param name Optional service name can be specified. If service name is specified then this service can only be
	 *              retrieved by a service name. If no service name is specified then service can be retrieved by its type
	 */
	export function Service(name?: string): (target: Function) => void;
	/**
	 * Injects a service into class property or into constructor parameter.
	 */
	export function Inject(type?: Function): Function;
	export function Inject(serviceName?: string): Function;
	/**
	 * Makes a "require" npm package with the given name and injects its value.
	 *
	 * @param name The name of the package to require
	 */
	export function Require(name: string): (target: any, key: string, index?: number) => void;

}
